@@ -2,10 +2,13 @@
 
 #include <glade/glade.h>
 
-#include "gva-game-db.h"
+#include "gva-error.h"
 #include "gva-favorites.h"
+#include "gva-game-db.h"
 #include "gva-game-store.h"
+#include "gva-main.h"
 #include "gva-play-back.h"
+#include "gva-preferences.h"
 #include "gva-tree-view.h"
 #include "gva-util.h"
 #include "gva-xmame.h"
@@ -14,7 +17,6 @@ static GladeXML *xml = NULL;
 static GtkUIManager *manager = NULL;
 static GtkActionGroup *action_group = NULL;
 static gboolean initialized = FALSE;
-static guint menu_tooltip_cid = 0;
 
 /* About Dialog Information */
 static const gchar *authors[] =
@@ -24,6 +26,16 @@ static const gchar *authors[] =
 };
 static const gchar *copyright = "Copyright \xC2\xA9 2007 Matthew Barnes";
 
+static void
+record_game_exited (GvaProcess *process, gint status, gchar *inpname)
+{
+        if (process->error == NULL)
+                gva_play_back_show (inpname);
+
+        g_object_unref (process);
+        g_free (inpname);
+}
+
 static void
 action_about_cb (GtkAction *action)
 {
@@ -33,11 +45,7 @@ action_about_cb (GtkAction *action)
         logo = gtk_icon_theme_load_icon (
                 gtk_icon_theme_get_default (),
                 PACKAGE, 128, 0, &error);
-        if (error != NULL)
-        {
-                g_warning ("%s", error->message);
-                g_error_free (error);
-        }
+        gva_error_handle (&error);
 
         gtk_show_about_dialog (
                 GTK_WINDOW (GVA_WIDGET_MAIN_WINDOW),
@@ -54,11 +62,29 @@ action_about_cb (GtkAction *action)
                 g_object_unref (logo);
 }
 
+static void
+action_auto_save_cb (GtkToggleAction *action)
+{
+        gboolean active;
+
+        active = gtk_toggle_action_get_active (action);
+        gva_preferences_set_auto_save (active);
+}
+
 static void
 action_contents_cb (GtkAction *action)
 {
 }
 
+static void
+action_full_screen_cb (GtkToggleAction *action)
+{
+        gboolean active;
+
+        active = gtk_toggle_action_get_active (action);
+        gva_preferences_set_full_screen (active);
+}
+
 static void
 action_insert_favorite_cb (GtkAction *action)
 {
@@ -90,13 +116,15 @@ action_insert_favorite_cb (GtkAction *action)
 static void
 action_play_back_cb (GtkAction *action)
 {
+        GvaProcess *process;
         GtkTreeModel *model;
         GtkTreeView *view;
         GtkTreeIter iter;
-        gchar *inpname;
         gchar *inpfile;
+        gchar *inpname;
+        gchar *romname;
         GList *list;
-        gboolean iter_set;
+        gboolean valid;
         GError *error = NULL;
 
         view = GTK_TREE_VIEW (GVA_WIDGET_PLAY_BACK_TREE_VIEW);
@@ -108,23 +136,35 @@ action_play_back_cb (GtkAction *action)
                 gtk_tree_view_get_selection (view), &model);
         g_assert (g_list_length (list) == 1);
 
-        iter_set = gtk_tree_model_get_iter (model, &iter, list->data);
-        g_assert (iter_set);
+        valid = gtk_tree_model_get_iter (model, &iter, list->data);
+        g_assert (valid);
         gtk_tree_model_get (
-                model, &iter, GVA_GAME_STORE_COLUMN_INPFILE, &inpfile, -1);
+                model, &iter, GVA_GAME_STORE_COLUMN_INPFILE, &inpfile,
+                GVA_GAME_STORE_COLUMN_ROMNAME, &romname, -1);
         inpname = g_strdelimit (g_path_get_basename (inpfile), ".", '\0');
         g_free (inpfile);
 
-        if (!gva_xmame_playback_game (inpname, &error))
-        {
-                g_warning ("%s", error->message);
-                g_error_free (error);
-        }
+        process = gva_xmame_playback_game (romname, inpname, &error);
+        gva_error_handle (&error);
+
+        if (process != NULL)
+                g_signal_connect_after (
+                        process, "exited",
+                        G_CALLBACK (g_object_unref), NULL);
+
+        g_free (inpname);
+        g_free (romname);
 
         g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
         g_list_free (list);
 }
 
+static void
+action_preferences_cb (GtkAction *action)
+{
+        gtk_widget_show (GVA_WIDGET_PREFERENCES_WINDOW);
+}
+
 static void
 action_properties_cb (GtkAction *action)
 {
@@ -139,6 +179,7 @@ action_quit_cb (GtkAction *action)
 static void
 action_record_cb (GtkAction *action)
 {
+        GvaProcess *process;
         const gchar *romname;
         gchar *inpname;
         GError *error = NULL;
@@ -147,14 +188,14 @@ action_record_cb (GtkAction *action)
         g_assert (romname != NULL);
 
         inpname = gva_choose_inpname (romname);
-        gva_xmame_record_game (romname, inpname, &error);
-        g_free (inpname);
 
-        if (error != NULL)
-        {
-                g_warning ("%s", error->message);
-                g_clear_error (&error);
-        }
+        process = gva_xmame_record_game (romname, inpname, &error);
+        gva_error_handle (&error);
+
+        if (process != NULL)
+                g_signal_connect_after (
+                        process, "exited",
+                        G_CALLBACK (record_game_exited), inpname);
 }
 
 static void
@@ -188,25 +229,30 @@ action_remove_favorite_cb (GtkAction *action)
 static void
 action_show_play_back_cb (GtkAction *action)
 {
-        gtk_widget_show (GVA_WIDGET_PLAY_BACK_WINDOW);
+        gva_play_back_show (NULL);
 }
 
 static void
 action_start_cb (GtkAction *action)
 {
+        GvaProcess *process;
         const gchar *romname;
         GError *error = NULL;
 
         romname = gva_tree_view_get_selected_game ();
         g_assert (romname != NULL);
 
-        gva_xmame_run_game (romname, &error);
+        if (!gva_preferences_get_auto_save ())
+                if (!gva_xmame_clear_state (romname, &error))
+                        gva_error_handle (&error);
 
-        if (error != NULL)
-        {
-                g_warning ("%s", error->message);
-                g_clear_error (&error);
-        }
+        process = gva_xmame_run_game (romname, &error);
+        gva_error_handle (&error);
+
+        if (process != NULL)
+                g_signal_connect_after (
+                        process, "exited",
+                        G_CALLBACK (g_object_unref), NULL);
 }
 
 static void
@@ -247,11 +293,18 @@ static GtkActionEntry entries[] =
           N_("Play back the selected game recording"),
           G_CALLBACK (action_play_back_cb) },
 
+        { "preferences",
+          GTK_STOCK_PREFERENCES,
+          N_("_Preferences"),
+          NULL,
+          N_("Configure the application"),
+          G_CALLBACK (action_preferences_cb) },
+
         { "properties",
           GTK_STOCK_PROPERTIES,
           N_("_Properties"),
           NULL,
-          N_("Show properties about the selected game"),
+          N_("Show information about the selected game"),
           G_CALLBACK (action_properties_cb) },
 
         { "quit",
@@ -289,6 +342,13 @@ static GtkActionEntry entries[] =
           N_("Start the selected game"),
           G_CALLBACK (action_start_cb) },
 
+        { "edit-menu",
+          NULL,
+          N_("_Edit"),
+          NULL,
+          NULL,
+          NULL },
+
         { "game-menu",
           NULL,
           N_("_Game"),
@@ -311,6 +371,25 @@ static GtkActionEntry entries[] =
           NULL }
 };
 
+static GtkToggleActionEntry toggle_entries[] =
+{
+        { "auto-save",
+          NULL,
+          N_("_Restore previous state when starting a game"),
+          NULL,
+          NULL,
+          G_CALLBACK (action_auto_save_cb),
+          FALSE },  /* GConf overrides this */
+
+        { "full-screen",
+          NULL,
+          N_("Start games in _fullscreen mode"),
+          NULL,
+          NULL,
+          G_CALLBACK (action_full_screen_cb),
+          FALSE }   /* GConf overrides this */
+};
+
 static GtkRadioActionEntry view_radio_entries[] =
 {
         { "view-available",
@@ -335,55 +414,6 @@ static GtkRadioActionEntry view_radio_entries[] =
           2 }
 };
 
-static void
-ui_menu_item_select_cb (GtkItem *item, GtkAction *action)
-{
-        GtkStatusbar *statusbar;
-        gchar *tooltip;
-
-        statusbar = GTK_STATUSBAR (GVA_WIDGET_MAIN_STATUSBAR);
-        g_object_get (G_OBJECT (action), "tooltip", &tooltip, NULL);
-
-        if (tooltip != NULL)
-        {
-                menu_tooltip_cid = gtk_statusbar_get_context_id (
-                        statusbar, G_STRFUNC);
-                gtk_statusbar_push (statusbar, menu_tooltip_cid, tooltip);
-                g_free (tooltip);
-        }
-}
-
-static void
-ui_menu_item_deselect_cb (GtkItem *item)
-{
-        GtkStatusbar *statusbar;
-
-        statusbar = GTK_STATUSBAR (GVA_WIDGET_MAIN_STATUSBAR);
-
-        if (menu_tooltip_cid != 0)
-        {
-                gtk_statusbar_pop (statusbar, menu_tooltip_cid);
-                menu_tooltip_cid = 0;
-        }
-}
-
-static void
-ui_manager_connect_proxy_cb (GtkUIManager *dont_shadow_global,
-                             GtkAction *action,
-                             GtkWidget *proxy)
-{
-        /* Make GtkMenuItems show their tooltips in the statusbar. */
-        if (GTK_IS_MENU_ITEM (proxy))
-        {
-                g_signal_connect (
-                        proxy, "select",
-                        G_CALLBACK (ui_menu_item_select_cb), action);
-                g_signal_connect (
-                        proxy, "deselect",
-                        G_CALLBACK (ui_menu_item_deselect_cb), NULL);
-        }
-}
-
 static void
 gva_ui_init (void)
 {
@@ -397,7 +427,11 @@ gva_ui_init (void)
 
         action_group = gtk_action_group_new ("main");
         gtk_action_group_add_actions (
-                action_group, entries, G_N_ELEMENTS (entries), NULL);
+                action_group, entries,
+                G_N_ELEMENTS (entries), NULL);
+        gtk_action_group_add_toggle_actions (
+                action_group, toggle_entries,
+                G_N_ELEMENTS (toggle_entries), NULL);
         gtk_action_group_add_radio_actions (
                 action_group, view_radio_entries,
                 G_N_ELEMENTS (view_radio_entries),
@@ -420,7 +454,7 @@ gva_ui_init (void)
 
                 g_signal_connect (
                         manager, "connect-proxy",
-                        G_CALLBACK (ui_manager_connect_proxy_cb), NULL);
+                        G_CALLBACK (gva_main_connect_proxy_cb), NULL);
 
                 gtk_ui_manager_add_ui_from_file (manager, filename, &error);
         }
